#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['django_mini_fastapi',
 'django_mini_fastapi.fastapi',
 'django_mini_fastapi.fastapi.dependencies',
 'django_mini_fastapi.fastapi.openapi']

package_data = \
{'': ['*']}

setup(name='django-mini-fastapi',
      version='0.1.7',
      description='Minimal FastAPI implementation for Django',
      author=None,
      author_email='toki <toki.kanno@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      python_requires='>=3.6',
     )
